import java.awt.AWTException;
import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
/**/
public class MainWindow extends JFrame implements KeyListener {
  /**/
  private ArrayList<String> helpFile;
  private CardRows crs;
  private CardResources cr;
  private ColumnDecksPanel cdp;
  private DestinationDeck [] dda;
  private DestinationDeckPanel ddp;
  private MainMenu mm;
  private MainPanel mp;
  private MoveProcessor move;
  private PlayDeck pd;
  private PlayDeckPanel pdp;
  private Robot robot;
  /**/
  public MainWindow ( String title ) {
    /**/
    super(title);
    /**/
    Deck d;
    JMenuBar mb;
    /**/
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setLocation(200,50);
    /**/
    cr=new CardResources();
    d=new Deck(cr);
    /**/
    robot=null;
    try { robot=new Robot(); }
    catch ( AWTException awte ) { }
    /**/
    crs=new CardRows(d);
    pd=new PlayDeck(d);
    dda=new DestinationDeck [4];
    for ( int i=0; i<4; ++i ) dda[i]=new DestinationDeck(i);
    /**/
    cdp=new ColumnDecksPanel(this,crs);
    pdp=new PlayDeckPanel(this,cr);
    ddp=new DestinationDeckPanel(this,cr);
    move=new MoveProcessor(this);
    /**/
    mm=new MainMenu();
    mb=new JMenuBar();
    mb.add(mm);
    setJMenuBar(mb);
    /**/
    mp=new MainPanel(ddp,pdp,cdp);
    getContentPane().add(mp);
    /**/
    addKeyListener(this);
    /**/
    initHelpFile();
  }
  /**/
  public CardResources getCardResources () {
    return cr;
  }
  /**/
  public CardRows getCardRows () {
    return crs;
  }
  /**/
  public ColumnDecksPanel getColumnDecksPanel () {
    return cdp;
  }
  /**/
  public PlayDeck getPlayDeck () {
    return pd;
  }
  /**/
  public PlayDeckPanel getPlayDeckPanel () {
    return pdp;
  }
  /**/
  public DestinationDeck [] getDestinationDecks () {
    return dda;
  }
  /**/
  public DestinationDeckPanel getDestinationDeckPanel () {
    return ddp;
  }
  /**/
  public MainPanel getMainPanel () {
    return mp;
  }
  /**/
  public MainMenu getMainMenu () {
    return mm;
  }
  /**/
  public MoveProcessor getMoveProcessor () {
    return move;
  }
  /**/
  public ArrayList<String> getHelpFile () {
    return helpFile;
  }
  /**/
  public Robot getRobot () {
    return robot;
  }  
  /**/
  public void keyPressed ( KeyEvent ke ) {
    return;
  }
  /**/
  public void keyReleased ( KeyEvent ke ) {
    /**/
    int code;
    InfoDialog mid;
    String[] info;
    /**/
    code=ke.getKeyCode();
    if ( code == KeyEvent.VK_ENTER ) {
      move.doMove();
      if ( gameIsWon() ) {
        info=new String [] {
          "              " ,
          " Game is won! " ,
          "              "
        };
        mid=new InfoDialog(this,"Win!",info,true);
        mid.popUp();
      }
    }
    /**/
    return;
  }
  /**/
  public void keyTyped ( KeyEvent ke ) {
    return;
  }
  /**/
  public void newGame () {
    /**/
    Deck d;
    /**/
    move.reset();
    d=new Deck(cr);
    crs=new CardRows(d);
    cdp.newGame(crs,cr);
    pd=new PlayDeck(d);
    pdp.newGame(cr);
    dda=new DestinationDeck [4];
    for ( int i=0; i<4; ++i ) dda[i]=new DestinationDeck(i);
    ddp.newGame(cr);
    /**/
    return;
  }
  /**/
  private void initHelpFile () {
    /**/
    BufferedReader br;
    InputStream is;
    String line;
    /**/
    helpFile=new ArrayList<String> ();
    /**/
    is=MainWindow.class.getResourceAsStream("help.txt");
    br=new BufferedReader( new InputStreamReader(is) );
    while ( true ) {
      line=readLine(br);
      if ( line == null ) break;
      helpFile.add(line);
    }
    close(br);
    /**/
    return;
  }
  /**/
  private boolean gameIsWon () {
    /**/
    return crs.haveNoDown()
           && ( pd.size() == 0 )
           && dda[0].containsAce()
           && dda[1].containsAce()
           && dda[2].containsAce()
           && dda[3].containsAce();
  }
  /**/
  private static String readLine ( BufferedReader br ) {
    /**/
    String line;
    /**/
    line=null;
    try { line=br.readLine(); }
    catch ( IOException ioe ) { }
    /**/
    return line;
  }
  /**/
  private static void close ( BufferedReader br ) {
    try { br.close(); }
    catch ( IOException ioe ) { }
  }
}
