import java.awt.image.BufferedImage;
/**/
public class MoveProcessor {
  /**/
  private int numClicks;
  private boolean fromIsSet;
  private MainWindow mw;
  private String fromSource, toSource;
  /**/
  public MoveProcessor ( MainWindow mw ) {
    /**/
    this.mw=mw;
    reset();
  }
  /**/
  public int getNumClicks () {
    return numClicks;
  }
  /**/
  public void incrementNumClicks () {
    ++numClicks;
  }
  /**/
  public void resetNumClicks () {
    numClicks=0;
  }
  /**/
  public boolean fromIsSet () {
    return fromIsSet;
  }
  /**/
  public void setFrom ( boolean b ) {
    fromIsSet=b;
  }
  /**/
  public void setFromSource( String s ) {
    fromSource=s;
  }
  /**/
  public void setToSource( String s ) {
    toSource=s;
  }
  /**/
  public boolean isReset () {
    return ( numClicks == 0 )
        && ( !fromIsSet )
        && ( fromSource == null )
        && ( toSource == null );
  }
  /**/
  public void reset () {
    numClicks=0;
    fromIsSet=false;
    fromSource=null;
    toSource=null;
  }
  /**/
  public void doMove () {
    /**/
    String [] error;
    String [] tokFrom, tokTo;
    /**/
    if ( fromSource == null ) {
      error = new String [] {
        "You must click on two cards" ,
        "to define the move."
      };
      error(error);
      return;
    }
    /**/
    tokFrom=fromSource.split(" ");
    /**/
    if ( toSource == null ) {
      error = new String [] {
        "You must click on another" ,
        "card to define the move."
      };
      error(error);
      resetFromPanel(tokFrom);
      return;
    }
    /**/
    tokTo=toSource.split(" ");
    /**/
    if      ( tokFrom[0].equals("PlayDeckPanel")        ) fromPlayDeck(tokFrom,tokTo);
    else if ( tokFrom[0].equals("DestinationDeckPanel") ) fromDestinationDeck(tokFrom,tokTo);
    else                                                  fromCardStack(tokFrom,tokTo);
    /**/
    return;
  }
  /**/
  private void resetPanels ( String [] from, String [] to ) {
    /**/
    resetPanel(from);
    resetPanel(to);
    reset();
    /**/
    return;
  }
  /**/
  private void resetFromPanel ( String [] from ) {
    /**/
    resetPanel(from);
    reset();
    /**/
    return;
  }
  /**/
  private void resetPanel ( String [] cmd ) {
    /**/
    int sp, len;
    BufferedImage bi;
    BufferedImage [] bia;
    Card card;
    CardResources cr;
    CardRow cRow;
    CardRows crs;
    CardStackPanel csp;
    CardStackPanel [] cspa;
    ColumnDecksPanel cdp;
    DestinationDeck dd;
    DestinationDeck [] dda;
    DestinationDeckPanel ddp;
    PlayDeck pd;
    PlayDeckPanel pdp;
    /**/
    cr=mw.getCardResources();
    pd=mw.getPlayDeck();
    pdp=mw.getPlayDeckPanel();
    dda=mw.getDestinationDecks();
    ddp=mw.getDestinationDeckPanel();
    crs=mw.getCardRows();
    cdp=mw.getColumnDecksPanel();
    /**/
    if      ( cmd[0].equals("PlayDeckPanel")        ) {
      if ( cmd[1].equals("up") ) {
        if ( pd.getUpDeck().size() == 0 ) bi=cr.getImage("empty");
        else                              bi=cr.getImage(pd.getTopCard().getName());
        pdp.getUpClickableCard().setImage(bi);
      }
      else {
        if ( pd.getDownDeck().size() == 0 ) bi=cr.getImage("empty");
        else                                bi=cr.getImage("back");
        pdp.getDownClickableCard().setImage(bi);
      }
    }
    else if ( cmd[0].equals("DestinationDeckPanel") ) {
      sp=Integer.parseInt(cmd[1].substring(0,1));
      dd=dda[sp];
      if ( !dd.containsAce() ) bi=cr.getImage("empty");
      else                     bi=cr.getImage(dd.getLastCard().getName());
      (ddp.getClickableCards())[sp].setImage(bi);
    }
    else {
      sp=Integer.parseInt(cmd[1].substring(0,1));
      cspa=cdp.getCardStackPanels();
      csp=cspa[sp];
      cRow=(crs.getCardRows())[sp];
      len=cRow.size();
      if ( len == 0 ) {
        bia=new BufferedImage [1];
        bia[0]=cr.getImage("empty");
        csp.setImages(bia);
      }
      else {
        bia=new BufferedImage[len];
        for ( int i=0; i<len; ++i ) {
          card=cRow.getCardFromIndex(i);
          if ( i != len-1 ) {
            if ( card.isUp() ) bia[i]=cr.getPartialImage(card.getName());
            else               bia[i]=cr.getPartialImage("back");
          }
          else {
            if ( !card.isUp() ) card.flip();
            bia[i]=cr.getImage(card.getName());
          }
        }
        csp.setImages(bia);
      }
    }
    /**/
    return;
  }
  /*
   *  from[0] == PlayDeckPanel
   */
  private void fromPlayDeck ( String [] from, String [] to ) {
    /**/
    int si, sc;
    Card card;
    CardRow cRow;
    CardRows crs;
    DestinationDeck dd;
    DestinationDeck [] dda;
    PlayDeck pd;
    String [] error;
    /**/
    pd=mw.getPlayDeck();
    dda=mw.getDestinationDecks();
    crs=mw.getCardRows();
    /*
     *  from[1] == up
     */
    if ( from[1].equals("up")   ) {
      if      ( to[0].equals("PlayDeckPanel")        ) {
        if ( to[1].equals("up") ) {
          error=new String [] {
            "Cannot move a card to itself."
          };
          error(error);
          resetFromPanel(from);
        }
        else {
          if ( pd.canFlip() ) {
            pd.flip();
            resetPanels(from,to);
          }
          else {
            error=new String [] {
              "Cannot flip."
            };
            error(error);
            resetPanels(from,to);
          }
        }
      }
      else if ( to[0].equals("DestinationDeckPanel") ) {
        si=Integer.parseInt(to[1].substring(0,1));
        if ( pd.getUpDeck().size() == 0 ) {
          error=new String [] {
            "The up play deck is empty."
          };
          error(error);
          resetPanels(from,to);
        }
        else {
          card=pd.getTopCard();
          if ( card.getSuit() != si ) {
            error = new String [] {
              "Cannot move card \"" + card.getName() + "\"" ,
              "to the selected destination deck."
            };
            error(error);
            resetPanels(from,to);
          }
          else {
            dd=dda[si];
            if ( !dd.canAdd(card) ) {
              error=new String [] {
                "Cannot put card \"" + card.getName() + "\"" ,
                "on the selected destination deck."
              };
              error(error);
              resetPanels(from,to);
            }
            else {
              dd.add(pd.remove());
              resetPanels(from,to);
            }
          }
        }
      }
      else {
        si=Integer.parseInt(to[1].substring(0,1));
        if ( pd.getUpDeck().size() == 0 ) {
          error=new String [] {
            "The up play deck is empty."
          };
          error(error);
          resetPanels(from,to);
        }
        else {
          sc=Integer.parseInt(to[1].substring(1));
          cRow=(crs.getCardRows())[si];
          card=pd.getTopCard();
          if ( cRow.canAdd(card) ) {
            cRow.add(pd.remove());
            resetPanels(from,to);
          }
          else {
            error=new String [] {
              "Cannot add card \"" + card.getName() + "\"" ,
              "to the selected column deck."
            };
            error(error);
            resetPanels(from,to);
          }
        }
      }
    }
    /*
     *  from[1] == down
     */
    else {
      if      ( to[0].equals("PlayDeckPanel")        ) {
        if ( to[1].equals("up") ) {
          if ( pd.canDraw() ) {
            pd.draw();
            resetPanels(from,to);
          }
          else {
            error=new String [] {
              "Cannot draw."
            };
            error(error);
            resetPanels(from,to);
          }
        }
        else {
          error=new String [] {
            "Cannot move a card to itself."
          };
          error(error);
          resetFromPanel(from);
        }
      }
      else if ( to[0].equals("DestinationDeckPanel") ) {
        error=new String [] {
          "Cannot move a card from the down" ,
          "play deck to a destination deck."
        };
        error(error);
        resetPanels(from,to);
      }
      else {
        error=new String [] {
          "Cannot move a card from the down" ,
          "play deck to a column deck."
        };
        error(error);
        resetPanels(from,to);
      }
    }
    /**/
    return;
  }
  /**/
  private void fromDestinationDeck ( String [] from, String [] to ) {
    /**/
    int fddn, si;
    Card cta;
    CardRow cr;
    CardRow [] cra;
    CardRows crs;
    DestinationDeck dd;
    DestinationDeck [] dda;
    String [] error;
    /**/
    crs=mw.getCardRows();
    dda=mw.getDestinationDecks();
    /**/
    if      ( to[0].equals("PlayDeckPanel")        ) {
      error=new String [] {
        "Cannot move a card from a destination" ,
        "deck to the play deck."
      };
      error(error);
    }
    else if ( to[0].equals("DestinationDeckPanel") ) {
      error=new String [] {
        "Cannot move a card from a destination" ,
        "deck to another destination deck."
      };
      error(error);
    }
    else {
      fddn=Integer.parseInt(from[1]);
      dd=dda[fddn];
      if ( dd.isEmpty() ) {
        error=new String [] {
          "The selected destination deck is empty."
        };
        error(error);
      }
      else {
        si=Integer.parseInt(to[1].substring(0,1));
        cra=crs.getCardRows();
        cr=cra[si];
        cta=dd.getLastCard();
        if ( !cr.canAdd(cta) ) {
          error=new String [] {
            "Cannot add card \"" + cta.getName() + "\"" ,
            "to the selected column deck."
          };
          error(error);
        }
        else {
          cr.add(dd.remove());
        }
      }
    }
    resetPanels(from,to);
    /**/
    return;
  }
  /**/
  private void fromCardStack ( String [] from, String [] to ) {
    /**/
    int fromCol, fromCrd, ddn, toCol;
    Card cta;
    CardRow cr, crDest;
    CardRow [] cra;
    CardRows crs;
    DestinationDeck dd;
    DestinationDeck [] dda;
    String [] error;
    /**/
    crs=mw.getCardRows();
    dda=mw.getDestinationDecks();
    /**/
    cra=crs.getCardRows();
    fromCol=Integer.parseInt(from[1].substring(0,1));
    fromCrd=Integer.parseInt(from[1].substring(1));
    cr=cra[fromCol];
    if ( cr.size() == 0 ) {
      error=new String [] {
        "The selected from column deck is empty."
      };
      error(error);
    }
    else {
      if      ( to[0].equals("PlayDeckPanel")        ) {
        error=new String [] {
          "Cannot move a card from a column" ,
          "deck to the play deck."
        };
        error(error);
      }
      else if ( to[0].equals("DestinationDeckPanel") ) {
        ddn=Integer.parseInt(to[1]);
        dd=dda[ddn];
        cta=cr.getCardFromIndex(fromCrd);
        if ( cta.isUp() ) {
          if ( cr.getReturnListSize(cta) != 1 ) {
            error=new String [] {
              "Cannot add multiple cards" ,
              "to a destination deck."
            };
            error(error);
          }
          else {
            if ( !dd.canAdd(cta) ) {
              error=new String [] {
                "Cannot add card \"" + cta.getName() + "\"" ,
                "to the selected destination deck."
              };
              error(error);
            }
            else {
              dd.add(cr.remove(cta).get(0));
            }
          }
        }
        else {
          error=new String [] {
            "Cannot add a down card" ,
            "to a destination deck."
          };
          error(error);
        }
      }
      else {
        toCol=Integer.parseInt(to[1].substring(0,1));
        if ( toCol == fromCol ) {
          error=new String [] {
            "Cannot move cards from a column" ,
            "deck to the same column deck."
          };
          error(error);
        }
        else {
          cta=cr.getCardFromIndex(fromCrd);
          if ( !cta.isUp() ) {
            error=new String [] {
              "Cannot add a down column deck" ,
              "card to another column deck."
            };
            error(error);
          }
          else {
            crDest=cra[toCol];
            if ( crDest.canAdd(cta) ) {
              crDest.add(cr.remove(cta));
            }
            else {
              error=new String [] {
                "Cannot add card \"" + cta.getName() + "\"" ,
                "to the selected column deck."
              };
              error(error);
            }
          }
        }
      }
    }
    resetPanels(from,to);
    /**/
    return;
  }
  /**/
  private void error ( String [] e ) {
    /**/
    int len;
    InfoDialog mid;
    String [] info;
    /**/
    len=e.length+2;
    info=new String [len];
    info[0]=" ";
    for ( int i=1; i<len-1; ++i ) info[i]= " " + e[i-1] + " ";
    info[len-1]=" ";
    /**/
    mid=new InfoDialog(mw,"Input Error",info,true);
    mid.popUp();
    /**/
    return;
  }
}
